@@ -47,13 +47,13 @@
 #include "um-user-manager.h"
 
 #include "um-account-dialog.h"
-#include "um-names-dialog.h"
 #include "um-language-dialog.h"
 #include "um-login-options.h"
 #include "um-password-dialog.h"
 #include "um-photo-dialog.h"
 #include "um-fingerprint-dialog.h"
 #include "um-utils.h"
+#include "um-strength-bar.h"
 
 #include "gdm-languages.h"
 
@@ -66,7 +66,6 @@ typedef struct {
         GtkWidget *language_chooser;
 
         UmAccountDialog *account_dialog;
-        UmNamesDialog *names_dialog;
         UmPasswordDialog *password_dialog;
         UmPhotoDialog *photo_dialog;
         UmLoginOptions *login_options;
@@ -269,9 +268,13 @@ user_changed (UmUserManager *um, UmUser *user, UserAccountDialog *d)
                                             -1);
                         g_object_unref (pixbuf);
                         g_free (text);
+                        g_object_unref (current);
 
                         break;
                 }
+                if (current)
+                        g_object_unref (current);
+
         } while (gtk_tree_model_iter_next (model, &iter));
 
         if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -280,6 +283,8 @@ user_changed (UmUserManager *um, UmUser *user, UserAccountDialog *d)
                 if (current == user) {
                         show_user (user, d);
                 }
+                if (current)
+                        g_object_unref (current);
         }
 }
 
@@ -357,15 +362,18 @@ delete_user_response (GtkWidget         *dialog,
 
         gtk_widget_destroy (dialog);
 
-        user = get_selected_user (d);
-
-        if (response_id == GTK_RESPONSE_NO) {
+        if (response_id == GTK_RESPONSE_CANCEL) {
+                return;
+        }
+        else if (response_id == GTK_RESPONSE_NO) {
                 remove_files = TRUE;
         }
         else {
                 remove_files = FALSE;
         }
 
+        user = get_selected_user (d);
+
         um_user_manager_delete_user (d->um,
                                      user,
                                      remove_files,
@@ -404,7 +412,7 @@ delete_user (GtkButton *button, UserAccountDialog *d)
                                                 um_user_get_real_name (user));
 
                 gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                          _("Deleting a user while he is logged in can leave the system in an inconsistent state."));
+                                                          _("Deleting a user while they are logged in can leave the system in an inconsistent state."));
                 g_signal_connect (dialog, "response",
                                   G_CALLBACK (gtk_widget_destroy), NULL);
         }
@@ -413,7 +421,7 @@ delete_user (GtkButton *button, UserAccountDialog *d)
                                                  0,
                                                  GTK_MESSAGE_QUESTION,
                                                  GTK_BUTTONS_NONE,
-                                                 _("Do you want to keep %s's files ?"),
+                                                 _("Do you want to keep %s's files?"),
                                                 um_user_get_real_name (user));
 
                 gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
@@ -425,7 +433,7 @@ delete_user (GtkButton *button, UserAccountDialog *d)
                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
                                         NULL);
 
-                gtk_window_set_icon_name (GTK_WINDOW (dialog), "system-config-users");
+                gtk_window_set_icon_name (GTK_WINDOW (dialog), "system-users");
 
                 g_signal_connect (dialog, "response",
                                   G_CALLBACK (delete_user_response), d);
@@ -480,6 +488,8 @@ show_user (UmUser *user, UserAccountDialog *d)
         gtk_label_set_text (GTK_LABEL (label), um_user_get_real_name (user));
         widget = get_widget (d, "full-name-button");
         gtk_widget_set_tooltip_text (label, um_user_get_user_name (user));
+        widget = get_widget (d, "full-name-entry");
+        gtk_entry_set_text (GTK_ENTRY (widget), um_user_get_real_name (user));
 
         label = get_widget (d, "account-type-value-label");
         gtk_label_set_text (GTK_LABEL (label), um_account_type_get_name (um_user_get_account_type (user)));
@@ -496,21 +506,23 @@ show_user (UmUser *user, UserAccountDialog *d)
                 }
         }  while (gtk_tree_model_iter_next (model, &iter));
 
-        switch (um_user_get_password_mode (user)) {
-        case UM_PASSWORD_MODE_REGULAR:
-                text = "\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2";
-                break;
-        case UM_PASSWORD_MODE_SET_AT_LOGIN:
-                text = C_("Password mode", "To be set at next login");
-                break;
-        case UM_PASSWORD_MODE_NONE:
-                text = C_("Password mode", "None");
-                break;
-        case UM_PASSWORD_MODE_DISABLED:
+        if (um_user_get_locked (user)) {
                 text = C_("Password mode", "Account disabled");
-                break;
-        default:
-                g_assert_not_reached ();
+        }
+        else {
+                switch (um_user_get_password_mode (user)) {
+                case UM_PASSWORD_MODE_REGULAR:
+                        text = "\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2\xe2\x80\xa2";
+                        break;
+                case UM_PASSWORD_MODE_SET_AT_LOGIN:
+                        text = C_("Password mode", "To be set at next login");
+                        break;
+                case UM_PASSWORD_MODE_NONE:
+                        text = C_("Password mode", "None");
+                        break;
+                default:
+                        g_assert_not_reached ();
+                }
         }
         label = get_widget (d, "account-password-value-label");
         gtk_label_set_text (GTK_LABEL (label), text);
@@ -599,17 +611,93 @@ selected_user_changed (GtkTreeSelection *selection, UserAccountDialog *d)
 }
 
 static void
-change_name (GtkButton *button, UserAccountDialog *d)
+name_style_set (GtkWidget *widget, GtkStyle *previous_style, UserAccountDialog *d)
 {
+        PangoFontDescription *desc;
+        gint size;
+
+        desc = pango_font_description_copy (widget->style->font_desc);
+        size = pango_font_description_get_size (desc);
+        pango_font_description_set_size (desc, 1.2 * size);
+        pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
+
+        g_signal_handlers_block_by_func (widget, name_style_set, d);
+        gtk_widget_modify_font (widget, desc);
+        g_signal_handlers_unblock_by_func (widget, name_style_set, d);
+
+        pango_font_description_free (desc);
+}
+
+static void
+change_name_start (GtkButton *button, UserAccountDialog *d)
+{
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "full-name-notebook")), 2);
+}
+
+static void
+change_name_done (GtkWidget         *entry,
+                  UserAccountDialog *d)
+{
+        const gchar *text;
         UmUser *user;
 
         user = get_selected_user (d);
 
-        um_names_dialog_set_user (d->names_dialog, user);
-        um_names_dialog_show (d->names_dialog,
-                              GTK_WINDOW (get_widget (d, "user-account-window")));
+        text = gtk_entry_get_text (GTK_ENTRY (entry));
+        if (g_strcmp0 (text, um_user_get_location (user)) != 0) {
+                um_user_set_real_name (user, text);
+        }
 
-        g_object_unref (user);
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "full-name-notebook")), 1);
+}
+
+static void
+change_name_canceled (UserAccountDialog *d)
+{
+        UmUser *user;
+        const gchar *text;
+        GtkWidget *widget;
+
+        user = get_selected_user (d);
+        text = um_user_get_real_name (user);
+
+        widget = get_widget (d, "full-name-value-label");
+        gtk_label_set_text (GTK_LABEL (widget), nonempty (text));
+        widget = get_widget (d, "full-name-button-label");
+        gtk_label_set_text (GTK_LABEL (widget), nonempty (text));
+        widget = get_widget (d, "full-name-entry");
+        gtk_entry_set_text (GTK_ENTRY (widget), text);
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "full-name-notebook")), 1);
+}
+
+static void
+change_name_activate (GtkWidget         *widget,
+                      UserAccountDialog *d)
+{
+        change_name_done (widget, d);
+}
+
+static gboolean
+change_name_focus_out (GtkWidget         *widget,
+                       GdkEventFocus     *event,
+                       UserAccountDialog *d)
+{
+        change_name_done (widget, d);
+
+        return FALSE;
+}
+
+static gboolean
+change_name_key_press (GtkWidget         *widget,
+                       GdkEventKey       *event,
+                       UserAccountDialog *d)
+{
+        if (event->keyval == GDK_Escape) {
+                change_name_canceled (d);
+        }
+
+        return FALSE;
 }
 
 static void
@@ -732,6 +820,27 @@ language_response (GtkDialog         *dialog,
         gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "account-language-notebook")), 1);
 }
 
+static gboolean
+finish_language_chooser (UserAccountDialog *d)
+{
+        GtkWidget *combo;
+
+        combo = get_widget (d, "account-language-combo");
+        d->language_chooser = um_language_chooser_new ();
+        gtk_window_set_transient_for (GTK_WINDOW (d->language_chooser),
+                                      GTK_WINDOW (d->main_window));
+        g_signal_connect (d->language_chooser, "response",
+                          G_CALLBACK (language_response), d);
+        g_signal_connect (d->language_chooser, "delete-event",
+                          G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+
+        gdk_window_set_cursor (gtk_widget_get_window (d->main_window), NULL);
+        gtk_window_present (GTK_WINDOW (d->language_chooser));
+        gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
+
+        return FALSE;
+}
+
 static void
 language_changed (GtkComboBox       *combo,
                   UserAccountDialog *d)
@@ -740,6 +849,7 @@ language_changed (GtkComboBox       *combo,
         GtkTreeIter iter;
         gchar *lang;
         UmUser *user;
+        GdkCursor *cursor;
 
         if (!gtk_combo_box_get_active_iter (combo, &iter))
                  return;
@@ -756,18 +866,18 @@ language_changed (GtkComboBox       *combo,
                 return;
         }
 
-        if (!d->language_chooser) {
-                d->language_chooser = um_language_chooser_new ();
-                gtk_window_set_transient_for (GTK_WINDOW (d->language_chooser),
-                                              GTK_WINDOW (d->main_window));
-                g_signal_connect (d->language_chooser, "response",
-                                  G_CALLBACK (language_response), d);
-                g_signal_connect (d->language_chooser, "delete-event",
-                                  G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+        if (d->language_chooser) {
+                gtk_window_present (GTK_WINDOW (d->language_chooser));
+                gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
+                return;
         }
 
-        gtk_window_present (GTK_WINDOW (d->language_chooser));
-        gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
+        cursor = gdk_cursor_new (GDK_WATCH);
+        gdk_window_set_cursor (gtk_widget_get_window (d->main_window),
+                               cursor);
+        gdk_cursor_unref (cursor);
+
+        g_idle_add ((GSourceFunc)finish_language_chooser, d);
 }
 
 static gboolean
@@ -1185,6 +1295,8 @@ lockbutton_changed (PolkitLockButton *button,
                 gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "account-password-notebook")), 0);
                 gtk_notebook_set_current_page (GTK_NOTEBOOK (get_widget (d, "account-fingerprint-notebook")), 0);
         }
+
+        um_password_dialog_set_privileged (d->password_dialog, is_authorized);
 }
 
 static void
@@ -1376,7 +1488,12 @@ setup_main_window (UserAccountDialog *d)
                            G_CALLBACK (show_tooltip_now), NULL);
 
         button = get_widget (d, "full-name-button");
-        g_signal_connect (button, "clicked", G_CALLBACK (change_name), d);
+        g_signal_connect (button, "clicked", G_CALLBACK (change_name_start), d);
+        button = get_widget (d, "full-name-entry");
+        g_signal_connect (button, "style-set", G_CALLBACK (name_style_set), d);
+        g_signal_connect (button, "activate", G_CALLBACK (change_name_activate), d);
+        g_signal_connect (button, "focus-out-event", G_CALLBACK (change_name_focus_out), d);
+        g_signal_connect (button, "key-press-event", G_CALLBACK (change_name_key_press), d);
 
         button = get_widget (d, "account-type-value-label");
         setup_tooltip_with_embedded_icon (button,
@@ -1503,6 +1620,7 @@ main (int argc, char *argv[])
         GError *error;
         const gchar *filename;
         GtkWidget *button;
+        volatile GType type;
         GOptionContext *context;
         static gboolean do_debug;
         static gboolean do_version;
@@ -1560,6 +1678,9 @@ main (int argc, char *argv[])
                 }
         }
 
+        /* register types that the builder might need */
+        type = um_strength_bar_get_type ();
+
         d->builder = gtk_builder_new ();
         d->um = um_user_manager_ref_default ();
 
@@ -1578,7 +1699,6 @@ main (int argc, char *argv[])
         setup_main_window (d);
         d->login_options = um_login_options_new (d->builder);
         d->account_dialog = um_account_dialog_new ();
-        d->names_dialog = um_names_dialog_new ();
         d->password_dialog = um_password_dialog_new ();
         button = get_widget (d, "user-icon-button");
         d->photo_dialog = um_photo_dialog_new (button, d->main_window);
